#include <iostream>
#include "Cluster.h"

using namespace std;

namespace cluster {

/**
	Print a Group
*/
ostream& operator<<(ostream& str, const Group& g)
{
	g.print(str);
	return str;
}

/**
	Print an observation, this is a terminal algorithm
	not like operator<<(Group)
*/
ostream& operator<<(ostream& str, const Observation& o)
{
	str << o.id;
	return str;
}



//------------------------------
// Group implementation
//


Group::Group(const Group& g)
  : observations(g.observations)
{
	//
}


void Group::add(Observation *n)
{
	observations.push_back(n);
}



void Group::erase(Observation *n)
{
	observations.remove(n);
}

unsigned Group::size()
{
	unsigned s = 0;
	list<Observation *>::iterator iter = observations.begin();
	for (;iter != observations.end(); ++iter)
		if (*iter) s += (*iter)->size();
	return s;
}

double Group::compact()
{
	return 1.0;

}


void Group::print(ostream& str) const
{
	list<Observation *>::const_iterator iter = observations.begin();
	str << " [ ";
	if (*iter) (*iter)->print(str); ++iter;
	for (;iter!=observations.end(); ++iter)
	{
		if (*iter)
		{
			str << ',';
			(*iter)->print(str);
		}
	}
	str << " ] ";
}


//------------------------------
// Cluster implementation
//

/**
	Don't delete groups because they still exist
	even if the cluster is delete !
*/
Cluster::~Cluster()
{
	//
}


void Cluster::add(Group *g)
{

	//if (g && std::find(groups.begin(), groups.end(), g) != groups.end())
	//	return;

	groups.push_back(g);
	GroupID id = groups.size() - 1;

	ptr2ID.insert(std::map<Group *,GroupID>::value_type(g,id));

	cluster.insert(cluster.begin(), id);

	//cout << *g << "  - " << id << endl;
}

void Cluster::add(GroupIterator iter)
{
	Group *ptr = iter.ptr();
	add(ptr);
}

void Cluster::remove(Group *g)
{
	// the searched Group is still in the tree
	if (g && std::find(groups.begin(), groups.end(), g) != groups.end())
	{
		GroupID id = ptr2ID[g];
		tree<GroupID>::iterator iter = std::find(cluster.begin(), cluster.end(), id);
		if (iter != cluster.end())
		{
			cluster.erase(iter);
			std::cout << "found => erasing" << endl;
			groups[id] = 0;
			std::cout << "found => erased" << endl;
		}
		else
			std::cout << "Cluster::remove - No " << id << " in the tree" << std::endl;
	}
	else
		std::cout << "Cluster::remove - The group doesn't exist in this cluster" << std::endl;
}

void Cluster::remove(GroupIterator iter)
{
	Group *ptr = iter.ptr();
	remove(ptr);
}



void Cluster::print(ostream& str) const
{
	str << "Cluster = {" << endl;
	vector<Group *>::const_iterator iter = groups.begin();
	for (;iter!=groups.end(); ++iter)
	{
		if (*iter)
		{
			(*iter)->print(str);
			str << endl;
		}
	}
	str << "}";

}


ostream& operator<<(ostream& str, const Cluster& c)
{
	c.print(str);
	return str;
}


}
