#ifndef VIEW_H
#define VIEW_H

#include <list>

#include "iterator.h"
#include "Cluster.h"
#include "DataEngine.h"

namespace cluster {


/**
	A View is a kind of screenshot of the clusterization
	at the current time
*/
class View
{
  public:
	typedef ClusterIterator      iterator;
	//typedef ClusterConstIterator const_iterator;

	// wanna do another kind of view ?
  protected:
	const database::DataEngine *engine;
	std::list<Cluster *>  view;

	//! Protected constructor...
	View(){}

  public:
	virtual ~View();
	View(const database::DataEngine& dataEngine)
	{
		engine = &dataEngine;
	}

	// customization of the view


	//! compute the global error of a view
	double computeRegressionModel();

	//! compute the global error of a view
	double computeGlobalError();

	//! compute the global error of a view
	double computeError(const Observation & obs, const Cluster & cluster);

	//! give a random float number between 0 and 1
	double rand() const { return engine->rand(); }

  //! give a random number between 0 and 1
	int rand( unsigned inf, unsigned sup) const
	{
		return engine->rand(inf,sup);
	}

	iterator randCluster();

	//! Size of the View (number of clusters)
	inline unsigned size() const {
		return view.size();
	}

	//! Add a cluster to the view
	inline View& add(Cluster& c) {
		// test the cluster we want to add is OK
		for (std::list<Cluster *>::iterator iter = view.begin(); iter != view.end(); ++iter)
		{
			if (&c == *iter)
				return *this;
		}
		view.push_back(&c);
		return *this;
	}

	//! Nice visu
	inline View& operator<<(Cluster& c) { return add(c); }


	// iterator information
	inline iterator begin()        { return ClusterIterator(view.begin()); }
	inline iterator end()          { return ClusterIterator(view.end());   }
	//inline const_iterator begin() const { return ClusterConstIterator(view.begin()); }
	//inline const_iterator end()   const { return ClusterConstIterator(view.end());   }

	// movment on view/clusters

	inline void move(ClusterIterator src , ClusterIterator dest, GroupIterator g) {
		dest->add(g);
		src->remove(g);
	}

	inline View& add(ClusterIterator c, const Group& g) {
		Group *ptr = const_cast<Group *>(&g);
		c->add(ptr);
		return *this;
	}
	inline View& add(ClusterIterator c, GroupIterator g) {
		c->add(g.ptr());
		return *this;
	}

	inline void remove(ClusterIterator c) {
		std::list<Cluster *>::iterator iter;
		if ((iter=std::find(view.begin(), view.end(), c.ptr())) != view.end())
		{
			*iter = 0;
		}
	}

	inline void remove(ClusterIterator c, const Group& g) {
		Group *ptr = &(const_cast<Group&>(g));
		c->remove(ptr);
	}

	inline void remove(ClusterIterator c, GroupIterator g) {
		c->remove(g.ptr());
	}

	friend std::ostream& operator<<(std::ostream& str, const View& v);

};


std::ostream& operator<<(std::ostream& str, const View& v);



}


#endif
