#ifndef OUR_ITERATOR_H
#define OUR_ITERATOR_H

#include <iterator>
#include <vector>
#include <list>

namespace cluster
{

class Cluster;
class Group;
class Observation;

/*

On peut pas utiliser un template car on peut pas faire un typename, ou passer un type
templatis en parametre...

template<class T>
class iterator_base
{
	typename std::vector<T *>::iterator current;
	typedef  std::vector<T *>::iterator type;

  public:
	iterator_base() {}
	iterator_base(const iterator_base& x)              : current(x.current) {}
	iterator_base(const type& x) : current(x)         {}
	~iterator_base() {}

	// operators
	inline iterator_base& operator++() { ++current; return *this; }
	inline iterator_base& operator--() { --current; return *this; }

	inline iterator_base  operator++(int) { iterator_base x(*this); ++*this; return x; }
	inline iterator_base  operator--(int) { iterator_base x(*this); --*this; return x; }

	inline T& operator*() const       { return *current; }

	inline bool operator==(const iterator_base& x) { return x.current == current; }
	inline bool operator!=(const iterator_base& x) { return x.current != current; }
};

*/

class ClusterIterator
{
	typedef  std::list<Cluster *>::iterator iterator_type;
	iterator_type current;

  public:
	typedef Cluster  value_type;
	typedef Cluster& reference;
	typedef Cluster* pointer;


  public:
	ClusterIterator() {}
	ClusterIterator(const ClusterIterator& x) : current(x.current) {}
	ClusterIterator(const iterator_type& x) : current(x) {}
	~ClusterIterator() {}

	inline ClusterIterator& operator=(const ClusterIterator& x) { current=x.current; return *this; }

	// operators
	inline ClusterIterator& operator++() { ++current; return *this; }
	inline ClusterIterator& operator--() { --current; return *this; }

	inline ClusterIterator  operator++(int) { ClusterIterator x(*this); ++*this; return x; }
	inline ClusterIterator  operator--(int) { ClusterIterator x(*this); --*this; return x; }

	inline Cluster& operator*() const { return **current; }
	inline Cluster& ref()       const { return **current; }
	inline Cluster* operator->()      { return *current;  }
	inline Cluster* ptr()             { return *current;  }

	inline bool operator==(const ClusterIterator& x) { return x.current == current; }
	inline bool operator!=(const ClusterIterator& x) { return x.current != current; }
};

/*
class ClusterConstIterator
{
	typedef std::list<Cluster *>::const_iterator iterator_type;
	iterator_type current;

  public:
	friend class ClusterIterator;


	typedef const Cluster  value_type;
	typedef const Cluster& reference;
	typedef const Cluster* pointer;


  public:
	ClusterConstIterator() {}
	ClusterConstIterator(const ClusterIterator& x) : current(x.current) {}
	ClusterConstIterator(const ClusterConstIterator& x) : current(x.current) {}
	ClusterConstIterator(const iterator_type& x) : current(x) {}
	~ClusterConstIterator() {}

	inline ClusterConstIterator& operator=(const ClusterConstIterator& x) { current=x.current; return *this; }

	// operators
	inline ClusterConstIterator& operator++() { ++current; return *this; }
	inline ClusterConstIterator& operator--() { --current; return *this; }

	inline ClusterConstIterator  operator++(int) { ClusterConstIterator x(*this); ++*this; return x; }
	inline ClusterConstIterator  operator--(int) { ClusterConstIterator x(*this); --*this; return x; }

	inline const Cluster& operator*() const { return **current; }
	inline const Cluster& ref()       const { return **current; }
	inline const Cluster* operator->()      { return *current;  }
	inline const Cluster* ptr()             { return *current;  }

	inline bool operator==(const ClusterConstIterator& x) { return x.current == current; }
	inline bool operator!=(const ClusterConstIterator& x) { return x.current != current; }
};

*/

class GroupIterator
{
	typedef std::vector<Group *>::iterator iterator_type;
	iterator_type current;

  public:
	typedef Group  value_type;
	typedef Group& reference;
	typedef Group* pointer;

  public:
	GroupIterator() {}
	GroupIterator(const GroupIterator& x) : current(x.current) {}
	GroupIterator(const iterator_type& x) : current(x)         {}
	~GroupIterator() {}

	inline GroupIterator& operator=(const GroupIterator& x) { current=x.current; return *this; }

	// operators
	inline GroupIterator& operator++() { ++current; return *this; }
	inline GroupIterator& operator--() { --current; return *this; }

	inline GroupIterator  operator++(int) { GroupIterator x(*this); ++*this; return x; }
	inline GroupIterator  operator--(int) { GroupIterator x(*this); --*this; return x; }

	inline Group& operator*() const       { return **current; }
	inline Group& ref()       const       { return **current; }
	inline Group* operator->()            { return *current;  }
	inline Group* ptr()                   { return *current;  }

	inline bool operator==(const GroupIterator& x) { return x.current == current; }
	inline bool operator!=(const GroupIterator& x) { return x.current != current; }
};


/**
	This is an observation iterator... a simple one
*/
class ObsIterator
{
	typedef std::list<Observation *>::iterator iterator_type;
	iterator_type current;

  public:
	typedef Observation  value_type;
	typedef Observation& reference;
	typedef Observation* pointer;

  public:
	ObsIterator() {}
	ObsIterator(const ObsIterator& x)   : current(x.current) {}
	ObsIterator(const iterator_type& x) : current(x)         {}
	~ObsIterator() {}

	inline ObsIterator& operator=(const ObsIterator& x) { current=x.current; return *this; }

	// operators
	inline ObsIterator& operator++() { ++current; return *this; }
	inline ObsIterator& operator--() { --current; return *this; }

	inline ObsIterator  operator++(int) { ObsIterator x(*this); ++*this; return x; }
	inline ObsIterator  operator--(int) { ObsIterator x(*this); --*this; return x; }

	inline Observation& operator*() const       { return **current; }
	inline Observation& ref()       const       { return **current; }
	inline Observation* operator->()            { return *current;  }
	inline Observation* ptr()                   { return *current;  }

	inline bool operator==(const ObsIterator& x) { return x.current == current; }
	inline bool operator!=(const ObsIterator& x) { return x.current != current; }
};


}

#endif
