#ifndef OUR_ITERATOR_H
#define OUR_ITERATOR_H

#include <iterator>
#include <vector>
#include <list>

namespace cluster
{

class Cluster;
class Group;

/*

On peut pas utiliser un template car on peut pas faire un typename, ou passer un type
templatis en parametre...

template<class T>
class iterator_base
{
	typename std::vector<T *>::iterator current;
	typedef  std::vector<T *>::iterator type;

  public:
	iterator_base() {}
	iterator_base(const iterator_base& x)              : current(x.current) {}
	iterator_base(const type& x) : current(x)         {}
	~iterator_base() {}

	// operators
	inline iterator_base& operator++() { ++current; return *this; }
	inline iterator_base& operator--() { --current; return *this; }

	inline iterator_base  operator++(int) { iterator_base x(*this); ++*this; return x; }
	inline iterator_base  operator--(int) { iterator_base x(*this); --*this; return x; }

	inline T& operator*() const       { return *current; }

	inline bool operator==(const iterator_base& x) { return x.current == current; }
	inline bool operator!=(const iterator_base& x) { return x.current != current; }
};

*/

class ClusterIterator
{
	typedef  std::vector<Cluster *>::iterator iterator_type;
	iterator_type current;

  public:
	typedef Cluster  value_type;
	typedef Cluster& reference;
	typedef Cluster* pointer;


  public:
	ClusterIterator() {}
	ClusterIterator(const ClusterIterator& x) : current(x.current) {}
	ClusterIterator(const iterator_type& x) : current(x) {}
	~ClusterIterator() {}

	inline ClusterIterator& operator=(const ClusterIterator& x) { current=x.current; return *this; }

	// operators
	inline ClusterIterator& operator++() { ++current; return *this; }
	inline ClusterIterator& operator--() { --current; return *this; }

	inline ClusterIterator  operator++(int) { ClusterIterator x(*this); ++*this; return x; }
	inline ClusterIterator  operator--(int) { ClusterIterator x(*this); --*this; return x; }

	inline Cluster& operator*() const { return **current; }
	inline Cluster& ref()       const { return **current; }
	inline Cluster* operator->()      { return *current;  }
	inline Cluster* ptre()            { return *current;  }

	inline bool operator==(const ClusterIterator& x) { return x.current == current; }
	inline bool operator!=(const ClusterIterator& x) { return x.current != current; }
};


class GroupIterator
{
	typedef std::vector<Group *>::iterator iterator_type;
	iterator_type current;

  public:
	typedef Group  value_type;
	typedef Group& reference;
	typedef Group* pointer;

  public:
	GroupIterator() {}
	GroupIterator(const GroupIterator& x) : current(x.current) {}
	GroupIterator(const iterator_type& x) : current(x)         {}
	~GroupIterator() {}

	inline GroupIterator& operator=(const GroupIterator& x) { current=x.current; return *this; }

	// operators
	inline GroupIterator& operator++() { ++current; return *this; }
	inline GroupIterator& operator--() { --current; return *this; }

	inline GroupIterator  operator++(int) { GroupIterator x(*this); ++*this; return x; }
	inline GroupIterator  operator--(int) { GroupIterator x(*this); --*this; return x; }

	inline Group& operator*() const       { return **current; }
	inline Group& ref()       const       { return **current; }
	inline Group* operator->()            { return *current;  }
	inline Group* ptr()                   { return *current;  }

	inline bool operator==(const GroupIterator& x) { return x.current == current; }
	inline bool operator!=(const GroupIterator& x) { return x.current != current; }
};




}

#endif
